/*
 *----------------------------------------------------------------------
 *
 * Copyright 1991, University of New Mexico.  All rights reserved.
 * Permission to copy and modify this software and its documen-
 * tation only for internal use in your organization is hereby
 * granted, provided that this notice is retained thereon and
 * on all copies.  UNM makes no representations as to the sui-
 * tability and operability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 * 
 * UNM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL UNM BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including, for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *---------------------------------------------------------------------
 */

#include "unmcopyright.h"        /* Copyright 1991 by UNM */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 >>>>
 >>>>         File Name: Fdat2viff.c
 >>>>
 >>>>      Program Name: Fdat2viff
 >>>>
 >>>>      Program Desc: 
 >>>>	
 >>>>	Conversion  from  Fremont   (4.7   and   7   tesla   magnet)
 >>>>	spectroscopy data (.DAT) file to VIFF format.
 >>>>	
 >>>>	
 >>>>            Author: Donna Koechner
 >>>>
 >>>> Date Last Updated: Mon Apr 15 22:31:31 1991
 >>>>
 >>>>          Routines: main- the main program for Fdat2viff
 >>>> 		 gw_usage - gives usage of the program
 >>>> 		 gw_args  - gets arguments of program from command line
 >>>>
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/


#include "Fdat2viff.h"


/****************************************************************
 *
 * Routine Name:  main program for Fdat2viff
 *
 *       Input:  
 *           -i  input raw .DAT data file 
 *
 *
 *           [-o1] complex output image (viff file)
 *           [-o2] real output image filename
 *           [-o3] imaginary output image filename
 *           [-r] number of rows  (value >= 0)
 *           [-c] number of columns  (value >= 0)
 *           [-b] number of points/voxel (bands)  (value > 0)
 *           [-h] byte offset from the start of the input data file  (value >= 0)
 *           [-s] Extract only one voxel vector (1 = yes, 0 = no)
 *           [-x] vertical offset of voxel  (value >= 0)
 *           [-y] horizontal offset of voxel  (value >= 0)
 *
 *
 *
 ****************************************************************/


main(argc, argv)
int argc;
char *argv[];
{

/* -main_variable_list */
struct xvimage  *img1, *img2, *img3;
int             single_voxel, header_offset, numbands, nrows, ncols;
int             col_offset, row_offset;
unsigned long   type;
char            *malloc();
int             file;
/* -main_variable_list_end */

	program = VStrcpy(argv[0]);
	ac = argc;
	av = argv;

	khoros_init();

	gw_get_args();

/* -main_before_lib_call */
if (check_args())
{
  fprintf(stderr, "%s: Illegal arguments on command line.\n", program);
  exit(1);
}

single_voxel = Fdat2viff->s_logic;
header_offset = Fdat2viff->h_int;
numbands = Fdat2viff->b_int;
nrows = Fdat2viff->r_int;
ncols = Fdat2viff->c_int;
row_offset = Fdat2viff->y_int;
col_offset = Fdat2viff->x_int;
type = VFF_TYP_4_BYTE;

if ( single_voxel && ((row_offset >= nrows) || (col_offset >= ncols)))
{
  (void) fprintf(stderr,"Fdat2viff: row or column offset out of range\n");
  exit(1);
}


if ( (!Fdat2viff->o1_flag) && (!Fdat2viff->o2_flag) && (!Fdat2viff->o3_flag) )
{
  (void) fprintf(stderr,"Fdat2viff:  specify at least one output file\n");
  exit(1);
}
/* -main_before_lib_call_end */

/* -main_library_call */
if ( (file = open_input(Fdat2viff->i_file)) < 0)
  {
    fprintf(stderr,"lFs122viff: Unable to access input file %s\n",Fdat2viff->i_file);
    exit(1);
  }

if(! lFdat2viff(file, &img1, nrows, ncols, header_offset, type, 
                 single_voxel, row_offset, col_offset, numbands))
{
  (void) fprintf(stderr, "sig2viff Failed\n");
  close(file);
  exit(1);
}
close(file);
/* -main_library_call_end */

/* -main_after_lib_call */
if (Fdat2viff->o2_flag)
{
  if ((img2=(struct xvimage *)malloc(sizeof(struct xvimage)))==NULL)
  {
    (void) fprintf(stderr,"Fdat2viff: malloc failed\n");
    exit(1);
  }
  if (!(copyheader(img1, img2)))           
  {
    (void) fprintf(stderr, "copyheader for %s Failed\n", Fdat2viff->o2_file);
    exit(1);
  }
  if (!(copyimagedata(img1, &img2, 1)))   
  {
    (void)fprintf(stderr, "copyimagedata for %s Failed\n", Fdat2viff->o2_file);
    exit(1);
  }
  if (!(lvctor(img2, 1)))
  {
    (void) fprintf(stderr, "lvctor for %s Failed\n", Fdat2viff->o2_file);
    exit(1);
  }
}

if (Fdat2viff->o3_flag)
{
  if ((img3=(struct xvimage *)malloc(sizeof(struct xvimage)))==NULL)
  {
    (void) fprintf(stderr,"Fdat2viff: malloc failed\n");
    exit(1);
  }
  if (!(copyheader(img1, img3)))
  {
    (void) fprintf(stderr, "copyheader for %s Failed\n", Fdat2viff->o3_file);
    exit(1);
  }
  if (!(copyimagedata(img1, &img3, 1)))  
  {
    (void)fprintf(stderr, "copyimagedata for %s Failed\n", Fdat2viff->o3_file);
    exit(1);
  }
  if (!(lvctor(img3, 2)))
  {
    (void) fprintf(stderr, "lvctor for %s Failed\n", Fdat2viff->o3_file);
    exit(1);
  }
}

if (Fdat2viff->o1_flag)
{
  writeimage(Fdat2viff->o1_file,img1);
}

if (Fdat2viff->o2_flag)
{
  writeimage(Fdat2viff->o2_file,img2);
}

if (Fdat2viff->o3_flag)
{
  writeimage(Fdat2viff->o3_file,img3);
}

/* -main_after_lib_call_end */


	khoros_close();
}


/****************************************************************
*
* Routine Name:  gw_usage 
*
*
* Purpose:  prints out the usage for Fdat2viff 
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_usage()
{
	fprintf(stderr, "Fdat2viff :\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\tConversion  from  Fremont   (4.7   and   7   tesla   magnet)\n");
	fprintf(stderr, "\tspectroscopy data (.DAT) file to VIFF format.\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");

/* -usage_additions */
(void) fprintf(stderr,"where \"-o1\" creates complex output data file\n");
(void) fprintf(stderr,"      \"-o2\" creates real output data file\n");
(void) fprintf(stderr,"      \"-o3\" creates imaginary output data file\n");
/* -usage_additions_end */

	fprintf(stderr,"\t-i  input raw .DAT data file  (infile)\n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\n");
	fprintf(stderr,"\t[-o1]  complex output image (viff file) (outfile) [null]\n");
	fprintf(stderr,"\t[-o2]  real output image filename (outfile) [null]\n");
	fprintf(stderr,"\t[-o3]  imaginary output image filename (outfile) [null]\n");
	fprintf(stderr,"\t[-r]  number of rows (integer, >= 0) [1]\n");
	fprintf(stderr,"\t[-c]  number of columns (integer, >= 0) [64]\n");
	fprintf(stderr,"\t[-b]  number of points/voxel (bands) (integer, > 0) [512]\n");
	fprintf(stderr,"\t[-h]  byte offset from the start of the input data file (integer, >= 0) [2048]\n");
	fprintf(stderr,"\t[-s]  Extract only one voxel vector (1 = yes, 0 = no) (boolean) [false]\n");
	fprintf(stderr,"\t[-x]  vertical offset of voxel (integer, >= 0) [0]\n");
	fprintf(stderr,"\t[-y]  horizontal offset of voxel (integer, >= 0) [0]\n");
	fprintf(stderr, " \n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\t[-V] Gives the version for Fdat2viff\n");
	fprintf(stderr, "\t[-U] Gives the usage for Fdat2viff\n");
	fprintf(stderr, "\t[-P] Prompts for command line options\n");
	fprintf(stderr, "\t[-A [file1]] Creates the answer file called Fdat2viff.ans or file1 \n");
	fprintf(stderr, "\t[-a [file1]] Uses Fdat2viff.ans or file1 as the answer file \n\n\n");

}


/****************************************************************
*
* Routine Name:  gw_getargs 
*
*
* Purpose:  gets arguments off command line for Fdat2viff
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_get_args()
{

char	*i_pstr = 
	"\nEnter: (i) input raw .DAT data file \n        {infile}: "; 
char	*o1_pstr = 
	"\nEnter: (o1) complex output image (viff file) \n        {outfile}: "; 
char	*o2_pstr = 
	"\nEnter: (o2) real output image filename \n        {outfile}: "; 
char	*o3_pstr = 
	"\nEnter: (o3) imaginary output image filename \n        {outfile}: "; 
char	*r_pstr = 
	"\nEnter: (r) number of rows \n        {integer, >= 0 [1] }: "; 
char	*c_pstr = 
	"\nEnter: (c) number of columns \n        {integer, >= 0 [64] }: "; 
char	*b_pstr = 
	"\nEnter: (b) number of points/voxel (bands) \n        {integer, > 0 [512] }: "; 
char	*h_pstr = 
	"\nEnter: (h) byte offset from the start of the input data file \n        {integer, >= 0 [2048] }: "; 
char	*s_pstr = 
	"\nEnter: (s) Extract only one voxel vector (1 = yes, 0 = no) \n        {boolean, (y/n)  [no] }: "; 
char	*x_pstr = 
	"\nEnter: (x) vertical offset of voxel \n        {integer, >= 0 [0] }: "; 
char	*y_pstr = 
	"\nEnter: (y) horizontal offset of voxel \n        {integer, >= 0 [0] }: "; 
char  answer_file[512], temp[512];



int	fid = -1,
	error, query, ok, count;
	int prompt_flag = FALSE;

	/*
	 * print version if necessary
	 */
	vgparml(fid, "-V", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    printf ("\n%s: %s: Version %d.%d\n\n",
	    "Fdat2viff", RELEASE_NAME, RELEASE_MAJOR, RELEASE_MINOR);
	    exit(1);
	}

	/*
	 * print usage if necessary
	 */
	vgparml(fid, "-U", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    gw_usage();
	    exit(1);
	}

	/*
	 * see if interactive prompting is desired
	 */
	vgparml(fid, "-P", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	    prompt_flag = TRUE;

	/*
	 * see if answer file is to be used
	 */
	vgparms(fid, "-a", "noprompt", answer_file, 0, 0, &error);
	if (error == 0)
	{
	     if (VStrlen(answer_file) == 0)
		sprintf(answer_file, "Fdat2viff.ans");
	     while (fid < 0)
	     {
		if ((fid = open(answer_file, O_RDONLY, 0666))== -1)
		{
		    fprintf(stderr, "Fdat2viff: can't open answer file \"%s\"",
			    answer_file); 
		    fprintf(stderr, "please re-enter: ");
		    (void) gets(answer_file);
		    if (VStrlen(answer_file) == 0)
			sprintf(answer_file, "Fdat2viff.ans");
		}

	     }

	}

	/*
	 * allocate the Fdat2viff structure
	 */
	Fdat2viff = (Fdat2viff_struct *) 
	       calloc((unsigned)1, (unsigned) sizeof (Fdat2viff_struct));

	/*
	 * get required arguments for Fdat2viff
	 */
	if (prompt_flag == TRUE)
	    fprintf(stderr,"\nRequired Arguments:\n\n");

	if (!(Fdat2viff->i_flag = vget_infile(fid, prompt_flag,
			"-i", i_pstr, &(Fdat2viff->i_file))
))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(Fdat2viff->i_flag))
		{
		    fprintf(stderr, "\t\ti is required, please re-enter: \n");
		    Fdat2viff->i_flag = vget_infile(fid, prompt_flag,
		    "-i", i_pstr, &(Fdat2viff->i_file))
;

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-i' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}

	/*
	 * get optional arguments for Fdat2viff
	 */
	if (prompt_flag == TRUE)
	   fprintf(stderr,"\nOptional Arguments:\n\n");

	Fdat2viff->o1_flag = vget_outfile(fid, prompt_flag,
	"-o1", o1_pstr, &(Fdat2viff->o1_file))
;

	Fdat2viff->o2_flag = vget_outfile(fid, prompt_flag,
	"-o2", o2_pstr, &(Fdat2viff->o2_file))
;

	Fdat2viff->o3_flag = vget_outfile(fid, prompt_flag,
	"-o3", o3_pstr, &(Fdat2viff->o3_file))
;

	Fdat2viff->r_flag = vget_int(fid, prompt_flag,
	"-r", r_pstr, &(Fdat2viff->r_int),
	1, 1, 1);

	Fdat2viff->c_flag = vget_int(fid, prompt_flag,
	"-c", c_pstr, &(Fdat2viff->c_int),
	64, 1, 1);

	Fdat2viff->b_flag = vget_int(fid, prompt_flag,
	"-b", b_pstr, &(Fdat2viff->b_int),
	512, 2, 2);

	Fdat2viff->h_flag = vget_int(fid, prompt_flag,
	"-h", h_pstr, &(Fdat2viff->h_int),
	2048, 1, 1);

	Fdat2viff->s_flag = vget_logic(fid, prompt_flag,
	"-s", s_pstr, &(Fdat2viff->s_logic), 0);

	Fdat2viff->x_flag = vget_int(fid, prompt_flag,
	"-x", x_pstr, &(Fdat2viff->x_int),
	0, 1, 1);

	Fdat2viff->y_flag = vget_int(fid, prompt_flag,
	"-y", y_pstr, &(Fdat2viff->y_int),
	0, 1, 1);


}



